declare module "vector" {
    export class Vector {
        x: number;
        y: number;
        constructor(x: number, y: number);
        static fromPolarCoordinates(length: number, angle: number): Vector;
        add(x: number, y: number): this;
        subVec(vector: Vector): this;
        clone(): Vector;
        length(): number;
        angle(): number;
        moveForward(distance: number): void;
        isEqual(vector: Vector): boolean;
        intersectionPoint(lineOrigin: Vector, lineTarget: Vector): Vector;
    }
}
declare module "abstract_node" {
    import { Vector } from "vector";
    export abstract class AbstractNode {
        id: string;
        left: number;
        right: number;
        top: number;
        bottom: number;
        get width(): number;
        get height(): number;
        get centerX(): number;
        get centerY(): number;
        protected root: Element;
        constructor(nodeXML: Element);
        generateBoundingBox(): Vector[];
        abstract render(svg: SVGSVGElement, offsetX: number, offsetY: number): any;
        protected handleLabel(svg: SVGGElement): void;
        setDimensions(svgNode: SVGGElement | SVGSVGElement | SVGTextElement | SVGRectElement): void;
        setPosition(svgNode: SVGSVGElement | SVGTextElement | SVGGElement, offsetX: number, offsetY: number): void;
    }
}
declare module "edge" {
    import { AbstractNode } from "abstract_node";
    export class Edge {
        id: string;
        source: string;
        target: string;
        private root;
        constructor(edgeXML: Element);
        render(svg: SVGSVGElement, source: AbstractNode, target: AbstractNode, offsetX: number, offsetY: number): void;
        private hasStartArrow;
        private hasEndArrow;
        private handleArrow;
        private getEdgeColor;
        private handleLineStyle;
        private castRay;
    }
}
declare module "generic_node" {
    import { AbstractNode } from "abstract_node";
    export class GenericNode extends AbstractNode {
        protected shape: Element;
        constructor(nodeXML: Element);
        render(svg: SVGSVGElement, offsetX: number, offsetY: number): void;
        protected setFill(svgNode: SVGSVGElement | SVGGElement | SVGRectElement): void;
        protected setBorder(svgNode: SVGSVGElement | SVGGElement | SVGRectElement): void;
    }
}
declare module "shape_node" {
    import { GenericNode } from "generic_node";
    import { Vector } from "vector";
    export class ShapeNode extends GenericNode {
        constructor(nodeXML: Element);
        generateBoundingBox(): Vector[];
        private isEllipse;
        render(svg: SVGSVGElement, offsetX: number, offsetY: number): void;
        private generateRegularPolygon;
        private generateRegularPolygonVector;
    }
}
declare module "svg_node" {
    import { AbstractNode } from "abstract_node";
    export class SVGNode extends AbstractNode {
        protected shape: Element;
        protected resources: Map<string, string>;
        constructor(nodeXML: Element, resources: Map<string, string>);
        render(svg: SVGSVGElement, offsetX: number, offsetY: number): void;
        protected setFill(svgNode: SVGSVGElement | SVGGElement | SVGRectElement): void;
        protected setBorder(svgNode: SVGSVGElement | SVGGElement | SVGRectElement): void;
    }
}
declare module "graphml-viewer" { }
//# sourceMappingURL=graphml-viewer.d.ts.map